#!/usr/bin/perl -w

use strict;
no strict 'vars';
use dialog;


# Make changes here
my $interactive = 1;
my $root_path = $ENV{'FS_UPDATE_ROOT'};
my $path_etc = "${root_path}/etc/finalscratch";
my $version_file = "${path_etc}/fs_version";
my $path_files = $root_path ? "/mnt/fat/fs_updates" : "/mnt/fs_disks/0_vfat/fs_updates";
my $path_cdrom = "/mnt/cdrom/fs_updates";
my $script_name = "update";
my @script_ext = ("sh", "pl", "py");

#my $redirect = '>/dev/null 2>&1';
my $redirect = '';

# Don't change these
my @files = ();
my @updates = ();
my %updates = ();
my $version = '';
my ($ver_major, $ver_minor, $ver_sub) = (0, 0, 0);
my ($high_major, $high_minor, $high_sub) = (0, 0, 0);
my ($next_major, $next_minor, $next_sub) = (0, 0, 0);

sub parse_version {
	if ($_[0] =~ /^(\d+)\.(\d+)\.(\d+)$/) {
		return ($1, $2, $3);
	} else {
		return (0, 0, 0);
	}
}

sub version_string {
	return "$_[0].$_[1].$_[2]";
}

sub load_version {
	$version = `cat $version_file`;
	chomp $version;
	($ver_major, $ver_minor, $ver_sub) = parse_version($version);
	$version = version_string($ver_major, $ver_minor, $ver_sub);
}

sub save_version {
	system("echo '" . version_string($ver_major, $ver_minor, $ver_sub) . "' > $version_file");
}

sub search_files {
	my $path = shift;
	opendir(DIR, $path) || return;
	my @dir_files = readdir(DIR);
	closedir DIR;
	foreach my $element (sort @dir_files) {
		if ($element =~ /^fs_update__/i) {
			push @files, join('/', $path, $element);
		}
	}
}

sub version_cmp {
	my ($a1, $a2, $a3, $b1, $b2, $b3) = @_;
	return -1 if ($a1 < $b1);
	return 1 if ($a1 > $b1);
	return -1 if ($a2 < $b2);
	return 1 if ($a2 > $b2);
	return -1 if ($a3 < $b3);
	return 1 if ($a3 > $b3);
	return 0;
}

sub version_stringcmp {
	my ($a1, $a2, $a3) = parse_version($a);
	my ($b1, $b2, $b3) = parse_version($b);
	return version_cmp($a1, $a2, $a3, $b1, $b2, $b3);
}

sub version_from {
	my $file = shift;
	return (0,0,0) unless $file =~ /fs_update__(\d+)_(\d+)_(\d+)__(\d+)_(\d+)_(\d+)/i;
	return ($1, $2, $3);
}

sub version_to {
	my $file = shift;
	return (0,0,0) unless $file =~ /fs_update__(\d+)_(\d+)_(\d+)__(\d+)_(\d+)_(\d+)/i;
	return ($4, $5, $6);
}


sub find_highest {
	my ($h1, $h2, $h3) = (0,0,0);
	foreach my $file (@files) {
		my ($v1, $v2, $v3) = version_to($file);
		next unless ($v1 or $v2 or $v3);
		if (version_cmp($h1, $h2, $h3, $v1, $v2, $v3) < 0) {
			($h1, $h2, $h3) = ($v1, $v2, $v3);
		}
	}
	return ($h1,$h2,$h3);
}

sub better_update {
	return (
		# prefer path_files over path_cdrom 
		(($_[0] =~ /^$path_files/) and ($_[1] !~ /^$path_files/)) 
		# prefer short over long filenames (i.e. dir over compressed file)
		or (length($_[0]) < length($_[1]))
	);
}

sub select_best_file {
	my $file = '';
	for (@_) {
		$file = $_ unless $file;
		if (better_update($_, $file)) {
			$file = $_;
		}
	}
	return $file;
}

sub find_version_to {
	my ($t1, $t2, $t3) = @_;
	my @found_files = ();
	foreach my $file (@files) {
		my ($v1, $v2, $v3) = version_to($file);
		next unless ($v1 or $v2 or $v3);
		if (version_cmp($t1, $t2, $t3, $v1, $v2, $v3) == 0) {
			push @found_files, $file;
		}
	}
	return select_best_file(@found_files);
}

sub find_version_from {
	my ($f1, $f2, $f3) = @_;
	my @found_files = ();
	foreach my $file (@files) {
		my ($v1, $v2, $v3) = version_from($file);
		next unless ($v1 or $v2 or $v3);
		if (version_cmp($f1, $f2, $f3, $v1, $v2, $v3) == 0) {
			push @found_files, $file;
		}
	}
	return select_best_file(@found_files);
}

sub select_files {
	my ($f1, $f2, $f3, $t1, $t2, $t3);
	@updates = ();
	($t1, $t2, $t3) = ($high_major, $high_minor, $high_sub);
	while (1) {
		my $file = find_version_to($t1, $t2, $t3);
		last unless $file;
		($f1, $f2, $f3) = version_from($file);
		($next_major, $next_minor, $next_sub) = ($f1, $f2, $f3);
		#print "Update from $f1.$f2.$f3 to $t1.$t2.$t3: $file\n";
		push @updates, $file;
		my $comp_to_current = version_cmp($f1, $f2, $f3, $ver_major, $ver_minor, $ver_sub);
		return if $comp_to_current == 0;
		last if $comp_to_current < 0;
		($t1, $t2, $t3) = ($f1, $f2, $f3);
	}
	@updates = ();
	return;
}

sub unpack_update {
	my $file = shift;
	my $path = '.';
	my $ext = '';
	my $target = '';
	if ($file =~ /^(.*)\/(.*)$/) {
		($path, $file) = ($1, $2);
	}
	if ($file =~ /^(.*?)(\..*)$/) {
		($file, $ext) = ($1, $2);
	}
	if ($path eq $path_cdrom) {
		$target = join('/', $path_files, $file);
		$file = join('/', $path_cdrom, $file);
		$path = $path_files;
	} else {
		$target = join('/', $path, $file);
	}
	return $target if -d $target;
	if ($ext =~ /\.tgz$|\.tar\.gz$/i) {
		print STDERR "cd '$path' ; tar xzf '${file}${ext}'\n";
		system("cd '$path' ; tar xzf '${file}${ext}' $redirect");
	} elsif ($ext =~ /\.tar.bz2$/i) {
		print STDERR "cd '$path' ; bzcat '${file}${ext}' | tar xf '-'\n";
		system("cd '$path' ; bzcat '${file}${ext}' | tar xf '-' $redirect");
	} elsif ($ext =~ /\.zip$/i) {
		print STDERR "cd '$path' ; unzip -o -q -L '${file}${ext}'\n";
		system("cd '$path' ; unzip -o -q -L '${file}${ext}' $redirect");
	} else {
		display_error("Unknown file format: ${file}${ext}");
		return '';
	}
	return $target;
}

sub find_script {
	my $base = shift;
	my $script = join('/', $base, $script_name);
	print STDERR "DEBUG: script base $script\n";
	return $script if -x $script;
	for my $ext (@script_ext) {
		return join ('.', $script, $ext) if -x join('.', $script, $ext);
	}
	return '';
}
		
sub install_update {
	my $file = shift;
	return 1 unless $file =~ /^(.*fs_update__\d+_\d+_\d+__\d+_\d+_\d+)/i;
	my $base = $1;
	my $script = find_script($base);
	unless ($script) {
		display_error("Could not locate update script.");
		return 1;
	}
	if ($script =~ /.pl$/) {
		print STDERR "perl $script\n";
		return system("perl $script");
	} elsif ($script =~ /.py$/) {
		print STDERR "python $script\n";
		return system("python $script");
	} else {
		print STDERR "sh $script\n";
		return system("sh $script");
	}
}

sub apply_update {
	my $file = shift;
	return 1 unless $file =~ /fs_update__(\d+)_(\d+)_(\d+)__(\d+)_(\d+)_(\d+)/;
	display_info("FinalScratch Update","Applying update from $1.$2.$3 to $4.$5.$6");
	my $dir = unpack_update($file);
	if (! -d $dir) {
		display_error("Update from $1.$2.$3 to $4.$5.$6 failed to unpack!");
		return 1;
	}
	if (install_update($dir)) {
		display_error("Update from $1.$2.$3 to $4.$5.$6 failed to install!");
		return 1;
	}
	($ver_major, $ver_minor, $ver_sub) = ($4, $5, $6);
	return 0;
}

sub select_updates {
	my $res = display_updates();
	
}

sub find_updates {
	%updates = ();
	foreach my $file (@files) {
		my ($v1, $v2, $v3) = version_to($file);
		next unless ($v1 or $v2 or $v3);
		my $v = version_string($v1, $v2, $v3);
		next if version_cmp($v1, $v2, $v3, $ver_major, $ver_minor, $ver_sub) <= 0;
		if (!$updates{$v} or better_update($file, $updates{$v})) {
			$updates{$v} = $file;
		}
	}
	if (%updates) {
		return display_updates();
	} else {
		display_error("No applicable updates were found! Current version: $version");
		return '';
	}
}

sub do_update {
	load_version();
	
	search_files($path_files);
	search_files($path_cdrom);
	search_files("/home/tim/project/fs_linux_cd/upgrade_scripts/testupdate");

	if ($interactive) {
		($high_major, $high_minor, $high_sub) = find_updates();
	} else {
		($high_major, $high_minor, $high_sub) = find_highest();
	}

	if ($high_major or $high_minor or $high_sub) {
		select_files();
		unless (@updates) {
			if (version_cmp($ver_major, $ver_minor, $ver_sub, $high_major, $high_minor, $high_sub)) {
				display_error("One or more updates missing! You need the file(s) to " .
					"update from version $ver_major.$ver_minor.$ver_sub to $next_major.$next_minor.$next_sub " .
					"in order to continue.");
			} else {
				display_error("YouAreAlreadyUpdated (this is never displayed)");
			}
		}
		while (my $file = pop @updates) {
			last if apply_update($file);
		}
		save_version();
	}
	my $new_version = version_string($ver_major, $ver_minor, $ver_sub);
	if ($version eq $new_version) {
		display_msg("FinalScratch Update", "Your system was not changed.");
	} else {
		display_msg("FinalScratch Update", "Your system was updated from version " .
			"$version to $new_version");
	}
}

do_update();




############### GUI #############

sub display_error {
	rhs_msgbox("ERROR", shift, 50);
}

sub display_msg {
	rhs_msgbox(shift, shift, 50);
}

sub display_yesno {
	rhs_yesno(shift, shift, 50);
}

sub display_info {
	rhs_infobox(shift, shift, 50);
}

sub display_text {
	rhs_textbox(shift, shift, 74, 21, "Continue");
}

sub display_clear {
	rhs_clear();
}

sub display_updates {

	my @choices = ();
	my $c = 0;
	my @u = sort version_stringcmp keys %updates;
	for my $k (@u) {
		my $update = $updates{$k};
		push @choices, 
			($c+1), 
			"Update to version $k", 
			($c == @u - 1);
		$c++;
	}

	my $s = rhs_radiolist("Select Update",
		"Current version is $version. " .
		"Please select the updates you want to install.", 
		60, 
		$c,
		@choices);

	if ($s) {
		return parse_version($u[$dialog_result - 1]);
	} else {
		return (0,0,0);
	}
}

