#!/usr/bin/perl

# STONE-IT  : fsXconf.pl
#
# change X configuration 
# author: fred descamps <descamps@stone-it.be>
# date: 22 MAY 2002
# rewrite: christian wolff <wolff@scara.com>
# date: 2002-07-03

require Xconfig;
require Xconfigurator;
use dialog;

$conf_file = "/etc/fsXconf";
$file_path = "/etc/X11/XF86Config-4";
$file_path_save = "/etc/X11/XF86Config-4.fs";
$log_file = "/tmp/fsXconf.log";

foreach $c_arg (@ARGV) {
	$reconfig = 1 if $c_arg eq "-r";
	$debug = 1 if $c_arg eq "-d";
}

check_todo();

#$choice = display_menu();
$choice = 1;
if ($choice eq "1") {
	do_autoinst();
} elsif ($choice eq "2") {
	do_manualinst();
}
exit;

sub do_manualinst {
	$did = system("xf86config");
	if ($did eq 0) {
		update_conf(); 
		system ("mv $file_path $file_path_save");
	}
}

sub putlog {
	$log_msg = @_;
	if ($debug) {
		open(FILE, ">>$log_file");
		print FILE "$log_msg\n";
		close FILE;
	}
}

sub update_conf {
	`echo '1' > $conf_file`;
}

sub check_todo {
	if ($reconfig) {
		if (-e "$file_path_save") {
			system("mv $file_path_save $file_path");
		}
	} else {
		my $done = `cat $conf_file 2>/dev/null`;
		chomp $done;
		exit if $done;
	}
}

sub select_keyboard {
	my ($keymap, $xkeymap) = display_keyboard_menu();
	install_kb($keymap, $xkeymap);
}

sub install_kb {
	my ($keymap, $xkeymap) = @_;
	my $data = undef;
	my $line= undef;
	my $oldkeyb = undef;
	
	my $err = 0;
	$err ||= system("loadkeys $keymap");
	$err ||= system("install-keymap $keymap");
	display_error("Keyboard map could not be loaded! Using default keymap.") if $err;
	 
	open(FILE, "$file_path");
	while (<FILE>) {
		$data .= $_;
		if (!(/^#/)) {
			$oldkeyb = $1 if(/\"XkbLayout\"\t\"(\w+)\"/g)
		}
	}

	$data =~ s/\"XkbLayout\"\t\"$oldkeyb\"/\"XkbLayout\"\t\"$xkeymap\"/;
	open(FILE, ">$file_path");
	print FILE $data;
	close(FILE);
}


sub do_autoinst {
	my ($cards) = Xconfigurator::cardConfigurationAuto();

	select_keyboard();

	if ($cards->{identifier}) {
		putlog("This card will be configured : $cards->{identifier}");
		putlog("Driver\t\"$cards->{driver}\" will be used !");
		$filecfg = `cat $file_path`;
		$filecfg =~ s/FS_VIDEO\"\n\s+Driver\s+\"\w+\"/FS_VIDEO\"\n\tDriver\t\t\"$cards->{driver}\"/;
		$filecfg =~ s/DefaultDepth\s+\d*/DefaultDepth\t24/;
		open(FILE,">$file_path");
		print FILE $filecfg;
		close FILE;
		update_conf();
# done by MJ: turning nvidia drivers off
#		if ($cards->{driver} eq "nv") { 
#			system("/usr/bin/nvidia on"); 
#		} else { 
			system("/usr/bin/nvidia off"); 
#		}
	} else {
		display_error("Could not configure your video card! " .
			"A standard driver (VESA-Mode) will be used. " .
			"This might result in a less than optimal performance " .
			"of your video card and/or monitor.");
		$filecfg = `cat $file_path`;
		$filecfg =~ s/FS_VIDEO\"\n\s+Driver\s+\"\w+\"/FS_VIDEO\"\n\tDriver\t\t\"vesa\"/;
		$filecfg =~ s/DefaultDepth\s+\d*/DefaultDepth\t16/;
		open(FILE, ">$file_path");
		print FILE $filecfg;
		close FILE;
		update_conf();
	}
}

sub display_menu {

	my @choices = ();
	my $i = 0;
	push @choices, 'A', 'Automatic'; $i++;
	push @choices, 'M', 'Manual'; $i++;

	my $s = rhs_menu("FinalScratch Installer",
		"Select detection method for your video card:", 
		60, 
		$i,
		@choices);

	if ($s and ($dialog_result eq 'M')) {
		return 2;
	}
	return 1;
}

sub display_keyboard_menu {

	my @keymaps = (
		['English (US)', 'us', 'us'],
		['Belgian', 'be-latin1', 'be'],
		['Brazilian', 'br-abnt2', 'br'],
		['Danish', 'dk', 'dk'],
		['Dutch', 'nl', 'nl'],
		['Dvorak', 'dvorak', 'dvorak'],
		['English (UK)', 'uk', 'uk'],
		['Finnish', 'fi', 'fi'],
		['French', 'fr', 'fr'],
		['German', 'de', 'de'],
		['Greek', 'gr', 'gr'],
		['Hebrew', 'hebrew', 'hebrew'],
		['Italian', 'it', 'it'],
		['Norwegian', 'no', 'no'],
		['Portugese', 'pt-latin1', 'pt'],
		['Spanish', 'es', 'es'],
		['Swedish', 'se', 'se']
	);

	my @choices = ();
	for (my $i = 0; $i < scalar(@keymaps); $i++) {
		push @choices, $i + 1, $keymaps[$i][0];
	}

	my $s = rhs_menu("FinalScratch Installer",
		"Select your keyboard layout:", 
		60, 
		scalar(@keymaps),
		@choices);

	if ($s && $dialog_result && (($dialog_result - 1) < scalar(@keymaps))) {
		return ($keymaps[$dialog_result - 1][1], $keymaps[$dialog_result - 1][2]);
	}
	return ("us","us");
}

sub display_error {
	rhs_msgbox("ERROR", shift, 50);
}

sub display_msg {
	rhs_msgbox(shift, shift, 50);
}

